@@ -7,9 +7,11 @@ Created on 4 Mar 2021
 While the NMEA 0183 © protocol is proprietary, the information here
 has been collated from public domain sources.
 
-:author: semuadmin
+:author: semuadmin (Steve Smith)
 """
 
+# pylint: disable=fixme
+
 from datetime import datetime
 
 INPUT = 1
@@ -407,13 +409,108 @@ NMEA_MSGIDS_PROP = {
     # ***************************************************************
     "MGNWPL": "Waypoint Location",
     # ***************************************************************
-    # Quectel Proprietary message types
+    # Quectel PAIR Proprietary message types LC29 etc.
     # ***************************************************************
+    "AIR001": "Ack",
+    "AIR002": "GNSS Subsys Power On",
+    "AIR003": "GNSS Subsys Power Off",
+    "AIR004": "GNSS Subsys Hot Start",
+    "AIR005": "GNSS Subsys Warm Start",
+    "AIR006": "GNSS Subsys Cold Start",
+    "AIR007": "GNSS Subsys Full Cold Start",
+    "AIR010": "Request Aiding",
+    # "AIR011": "Unknown definition",  # TODO
+    # "AIR020": "Get firmware release information",  # TODO
+    "AIR050": "Common Set Fix Rate",
+    "AIR051": "Common Get Fix Rate",
+    "AIR058": "Common Set Min Snr",
+    "AIR059": "Common Get Min Snr",
+    "AIR062": "Common Set NMEA Output Rate",
+    "AIR063": "Common Get NMEA Output Rate",
+    "AIR066": "Common Set GNSS Search Mode",
+    "AIR067": "Common Get GNSS Search Mode",
+    "AIR070": "Common Set Static Threshold",
+    "AIR071": "Common Get Static Threshold",
+    "AIR072": "Common Set Elev Mask",
+    "AIR073": "Common Get Elev Mask",
+    "AIR074": "Common Set AIC Enable",
+    "AIR075": "Common Get AIC Status",
+    "AIR080": "Common Set Navigation Mode",
+    "AIR081": "Common Get Navigation Mode",
+    "AIR086": "Common Set Debuglog Output",
+    "AIR087": "Common Get Debuglog Output",
+    "AIR100": "Common Set Nmea Output Mode",
+    "AIR101": "Common Get Nmea Output Mode",
+    "AIR104": "Common Set Dual Band",
+    "AIR105": "Common Get Dual Band",
+    # "AIR161": "Force standby mode",  # TODO
+    # "AIR351": "Support QZSS NMEA",  # TODO
+    "AIR382": "Test Lock System Sleep",
+    "AIR391": "Test Jamming Detect",
+    "AIR400": "DGPS Set Mode",
+    "AIR401": "DGPS Get Mode",
+    "AIR410": "SBAS Enable",
+    "AIR411": "SBAS Get Status",
+    "AIR420": "SLAS Enable",
+    "AIR421": "SLAS Get Status",
+    "AIR432": "RTCM Set Output Mode",
+    "AIR433": "RTCM Get Output Mode",
+    "AIR434": "RTCM Set Output Ant Pnt",
+    "AIR435": "RTCM Get Output Ant Pnt",
+    "AIR436": "RTCM Set Output Ephemeris",
+    "AIR437": "RTCM Get Output Ephemeris",
+    "AIR490": "Set EASYTM status",
+    "AIR491": "Get EASY status",
+    # "AIR496": "EPOC Enable", # TODO
+    # "AIR498": "EPOC Set Config",# TODO
+    # "AIR507": "EPOC Clear Data",# TODO
+    # "AIR508": "EPOC Get Status",# TODO
+    # "AIR509":"EPOC Get Prediction Status", # TODO
+    "AIR511": "NVRAM Save Navigation Data",
+    "AIR512": "NVRAM Clear Navigation Data",
+    "AIR513": "NVRAM Save Setting",
+    "AIR6010": "DR Set Custom Message Output",
+    "AIR6011": "DR Get Custom Message Output",
+    "AIR650": "Low Power Entry Rtc Mode",
+    # "AIR690": "Enter periodic mode for power saving",  # TODO
+    # "AIR751": "Fix NMEA output time behind PPS",  # TODO
+    "AIR752": "PPS Set Config Cmd",
+    # "AIR820": "Set LOCUS logging status",  # TODO
+    # "AIR821": "Get LOCUS logging status",  # TODO
+    # "AIR824": "Erase LOCUS logger flash",  # TODO
+    # "AIR826": "Dump LOCUS flash data",  # TODO
+    "AIR864": "IO Set Baudrate",
+    "AIR865": "IO Get Baudrate",
+    "AIR866": "IO Set Flow Control",
+    "AIR867": "IO Get Flow Control",
+    "AIRSPF": "Jamming status L1",
+    "AIRSPF5": "Jamming status L5",
+    # ***************************************************************
+    # Quectel PQTM Proprietary message types LG290P/LG580P
+    # ***************************************************************
+    "QTMANTENNASTATUS": "Report Antenna Status",
+    "QTMBKP": "Set Backup Mode",
+    "QTMCFGAIC": "Sets/Gets AIC Function",
+    "QTMCFGANTDELTA": "Sets/Gets Delta Between Ref Point and Antenna",
+    "QTMCFGANTINF": "Sets/Gets Antenna Information",
+    "QTMCFGBLD": "Sets/Gets Baseline Between Two Antennas",
     "QTMCFGCNST": "Sets/Gets Constellation Configuration",
+    "QTMCFGDR": "Sets/Gets DR State",
+    "QTMCFGDRHOT": "Sets/Gets DR Hot Start Feature",
+    "QTMCFGDRRTD": "Sets/Gets DR Running TIme and Distance",
+    "QTMCFGEINSMSG": "Sets/Gets INS, IMU, GPS Status",
+    "QTMCFGELETHD": "Sets/Gets Elevation Threshold for Position Engine",
     "QTMCFGFIXRATE": "Sets/Gets Fix Interval",
     "QTMCFGGEOFENCE": "Sets/Gets Geofence Feature",
-    "QTMCFGMSGRATE": "Sets/Gets Message Output Rate on Current Interface",
+    "QTMCFGGEOSEP": "Sets/Gets Geoidal Separation",
+    "QTMCFGIMUINT": "Sets/Gets Motion Trigger Interruption",
+    "QTMCFGIMUTC": "Sets/Gets IMU Temperature Compensation",
+    "QTMCFGLA": "Sets/Gets Lever Arm",
+    "QTMCFGLAM": "Sets/Gets Lever Arm Mode",
+    "QTMCFGMSGRATE": "Sets/Gets Message Output Rate",
+    "QTMCFGNAVMODE": "Sets/Gets Navigation Mode",
     "QTMCFGNMEADP": "Sets/Gets NMEA Precision",
+    "QTMCFGNMEATID": "Sets/Gets NMEA Talker ID",
     "QTMCFGODO": "Sets/Gets Odometer Feature",
     "QTMCFGPPS": "Sets/Gets PPS (Pulse Per Second) Feature",
     "QTMCFGPROT": "Sets/Gets I/O Protocol for Specified Port",
@@ -421,34 +518,106 @@ NMEA_MSGIDS_PROP = {
     "QTMCFGRSID": "Sets/Gets Reference Station ID",
     "QTMCFGRTCM": "Sets/Gets RTCM",
     "QTMCFGRTK": "Sets/Gets RTK Mode",  # Absolute, Relative
+    "QTMCFGRTKSRCTYPE": "Sets/Gets RTK Source Type",
+    "QTMMCFGRTKSRCTYPE": "Sets/Gets RTK Source Type",  # typo in LG580P firmware?
     "QTMCFGSAT": "Sets/Gets GNSS Satellite Mask",
-    "QTMCFGSIGNAL": "Sets/Gets GNSS Signal Mask",
+    "QTMCFGSBAS": "Sets/Gets SBAS",
+    "QTMCFGSIGGRP": "Sets/Gets GNSS Signal Group",
+    "QTMCFGSIGNAL": "Sets/Gets Antenna1 Signal Mask",
+    "QTMCFGSIGNAL2": "Sets/Gets Antenna2 Signal Mask",
+    "QTMCFGSTATICHOLD": "Sets/Gets Parking Static Hold",
     "QTMCFGSVIN": "Sets/Gets Survey-In Feature",  # Survey-In or Fixed
     "QTMCFGUART": "Configure UART Interface",
+    "QTMCFGVEHMOT": "Sets/Gets Vehicle Motion Detection",
+    "QTMCFGWN": "Sets/Gets Reference Start Week No",
     "QTMCOLD": "Cold Start",
     "QTMDEBUGOFF": "Disable Debug Log Output",
     "QTMDEBUGON": "Enable Debug Log Output",
     "QTMDOP": "Outputs Dilution of Precision",
+    "QTMDRCAL": "Dead Reckoning Calibration Status",
+    "QTMDRCLR": "Clear DR Calibration Data",
+    "QTMDRPVA": "DR Postion, Velosity and Attitude",
+    "QTMDRSAVE": "Save DR Calibration Data",
+    "QTMEOE": "Output End of Epoch Information",
     "QTMEPE": "Output Estimated Position Error",
     "QTMGEOFENCESTATUS": "Outputs Geofence Status",
+    "QTMGETUTC": "Get UTC Time Information",
     "QTMGNSSSTART": "Starts GNSS Engine",
     "QTMGNSSSTOP": "Stops GNSS Engine",
+    "QTMGPS": "Position Status in GNSS Mode",
     "QTMHOT": "Hot Start",
+    "QTMIMU": "IMU Raw Data",
+    "QTMIMUTYPE": "IMU Status",
+    "QTMINS": "Navigation Results",
+    "QTMJAMMINGSTATUS": "Jamming Detection Status",
+    "QTMLS": "Outputs Leap Second Forecast Information",
+    "QTMNAV": "Output Navigation Information",
     "QTMODO": "Outputs Odometer Information",
     "QTMPL": "Outputs Protection Level Information",
     "QTMPVT": "Outputs PVT (GNSS) Result",  # Position, Velocity, Track
+    "QTMQVER": "Queries Version Information",
     "QTMRESETODO": "Reset Odometer Distance",
     "QTMRESTOREPAR": "Restore to Default Values after Restart",
     "QTMSAVEPAR": "Save Configuration to Non-Volatile Memory",
+    "QTMSENMSG": "Sensor Information",
+    "QTMSN": "Queries Module Serial Number Code",
     "QTMSRR": "System Reset and Reboot",
+    "QTMSTD": "Outputs Time, Position, Velocoity and Attitude Deviations",
     "QTMSVINSTATUS": "Outputs Survey-In Status",
+    "QTMTAR": "Output Time and Attitude",
     "QTMTXT": "Outputs Short Text Message",  # Longer messages use multiple messages
     "QTMUNIQID": "Query Module Unique ID",
+    "QTMVEHATT": "Output Vehicle Attitude",
+    "QTMVEHMOT": "Output Vehicle Motion after DR Calibration",
+    "QTMVEHMSG": "Output Vehicle Information",
     "QTMVEL": "Output Velocity Information",
     "QTMVER": "Firmware Version",
     "QTMVERNO": "Query Firmware Version",
     "QTMWARM": "Warm Start",
     # ***************************************************************
+    # Quectel PSTM Proprietary message types (LG69T)
+    # ***************************************************************
+    "STMCOLD": "Perform cold start",  # Command
+    "STMCPU": "Output CPU usage and speed",  # Output
+    "STMDR1": "Report gyroscope and accelerometer details",  # Output
+    "STMDR2": "Report calibration status and availability",  # Output
+    "STMDRBSD": "Report IMU bias error estimates",  # Output
+    "STMDRCAL": "Report IMU and odometer calibration status",  # Output
+    "STMDRCONFID": "Report navigation and calibration error estimates",  # Output
+    "STMDRDEBUG": "Report DR debug information",  # Output
+    "STMDREPE": "Report DR horizontal position error estimates",  # Output
+    "STMDRGPS": "Report GNSS information",  # Output
+    "STMDRMMFB": "Set DR algorithm input data",  # Command
+    "STMDRMMFBKF": "Report MMFB data timing, acceptance, and usage",  # Output
+    "STMDRPVA": "Report PVT estimated by DR",  # Output
+    "STMDRPVASD": "Report DR PVT error estimates",  # Output
+    "STMDRSENMSG": "Output the IMU raw data",  # Output
+    "STMDRSINT": "Report sensor integration samples",  # Type:
+    "STMDRSTYPE": "Report DR type",  # Output
+    "STMDRSVF": "Report IMU vehicle dynamics",  # Output
+    "STMDRUPD": "Quectel internal debugging",  # Output
+    "STMEPHEM": "Load ephemeris into NVM",  # Command
+    "STMHOT": "Perform hot start",  # Command
+    "STMIMUSELFTESTCMD": "Execute IMU self-test",  # Command
+    "STMINITGPS": "Initialise GPS",  # Command #
+    "STMINITGPSERROR": "Initialise GPS Nak",  # Response
+    "STMINITGPSOK": "Initialise GPS Ack",  # Response
+    "STMINITTIME": "Initialise time",  # Command
+    "STMINITTIMEERROR": "Initialise time Nak",  # Response
+    "STMINITTIMEOK": "Initialise time Ack",  # Response
+    "STMPPSDATA": "Output the PPS data",  # Output
+    "STMRESTOREPAR": "Restore the factory setting parameters",  # Command
+    "STMSAVEPAR": "Save current configuration data block to NVM",  # Command
+    "STMSBAS": "Output SBAS satellite data",  # Output
+    "STMSETPAR": "Set parameter",  # Command
+    "STMSETPARERROR": "Set parameter Nak",  # Response
+    "STMSETPAROK": "Set parameter Ack",  # Response
+    "STMSRR": "Execute system reset and reboot",  # Command
+    "STMTEMP": "Report temperature",  # Output
+    "STMTG": "Output time and satellite information",  # Output
+    "STMTS": "Output tracked satellite parameters",  # Output
+    "STMWARM": "Perform warm start",  # Command
+    # ***************************************************************
     # Sepentrio X5 Proprietary message types
     # ***************************************************************
     "SSNHRP": "Heading, Roll, Pitch",
@@ -460,13 +629,13 @@ NMEA_MSGIDS_PROP = {
     # ***************************************************************
     # U-BLOX Proprietary message types
     # ***************************************************************
-    "UBX00": "PUBX-POSITION Lat/Long Position Data",
-    "UBX03": "PUBX-SVSTATUS Satellite Status",
-    "UBX04": "PUBX-TIME Time of Day and Clock Information",
+    "UBX00": "Lat/Long Position Data",
+    "UBX03": "Satellite Status",
+    "UBX04": "Time of Day and Clock Information",
     "UBX05": "Lat/Long Position Data",
     "UBX06": "Lat/Long Position Data",
-    "UBX40": "Set NMEA message output rate",
-    "UBX41": "PUBX-CONFIG Set Protocols and Baudrate",
+    "UBX40": "Set NMEA Message Output Rate",
+    "UBX41": "Set Protocols and Baudrate",
     # ***************************************************************
     # Trimble Proprietary message types
     # ***************************************************************
